﻿using ELTE.TicTacToeGame.Persistence;
using ELTE.TicTacToeGame.Model;
using ELTE.TicTacToeGame.ViewModel;
using ELTE.TicTacToeGame.View;

namespace ELTE.TicTacToeGame;

public partial class AppShell : Shell
{
    #region Private fields

    private ITicTacToeModel _gameModel;
    private TicTacToeViewModel _gameViewModel;

    private Store _store;
    private StoredGameBrowserModel _storeModel;
    private StoredGameBrowserViewModel _storeViewModel;

    private LoadGamePage _loadGamePage;
    private SaveGamePage _saveGamePage;

    #endregion

    #region Constructors

    public AppShell(ITicTacToeModel model)
	{
		InitializeComponent();

        _gameModel = model;
        _gameModel.GameWon += new EventHandler<GameWonEventArgs>(GameModel_GameWon);
        _gameModel.GameOver += new EventHandler(GameModel_GameOver);

        _gameViewModel = new TicTacToeViewModel(_gameModel);
        _gameViewModel.LoadGame += new EventHandler(GameViewModel_LoadGame); // kezeljük a nézetmodell eseményeit
        _gameViewModel.SaveGame += new EventHandler(GameViewModel_SaveGame);

        // a játékmentések kezelésének összeállítása
        _store = new Store(); // a perzisztencia betöltése az adott platformon
        _storeModel = new StoredGameBrowserModel(_store);
        _storeViewModel = new StoredGameBrowserViewModel(_storeModel);
        _storeViewModel.GameLoading += new EventHandler<StoredGameEventArgs>(StoreViewModel_GameLoading);
        _storeViewModel.GameSaving += new EventHandler<StoredGameEventArgs>(StoreViewModel_GameSaving);

        _loadGamePage = new LoadGamePage();
        _loadGamePage.BindingContext = _storeViewModel;

        _saveGamePage = new SaveGamePage();
        _saveGamePage.BindingContext = _storeViewModel;

        BindingContext = _gameViewModel;
    }

    #endregion

    #region Model event handlers

    /// <summary>
    /// Játék megnyerésének eseménykezelése.
    /// </summary>
    private async void GameModel_GameWon(object? sender, GameWonEventArgs e)
    {
        switch (e.Player)
        {
            case Player.PlayerO:
                await DisplayAlert("Játék vége!", "A kör játékos győzött!", "OK");
                break;
            case Player.PlayerX:
                await DisplayAlert("Játék vége!", "A kereszt játékos győzött!", "OK");
                break;
        }
        _gameModel.NewGame(); // új játékot kezdünk
    }

    /// <summary>
    /// Játék végének eseménykezelése.
    /// </summary>
    private async void GameModel_GameOver(object? sender, EventArgs e)
    {
        await DisplayAlert("Döntetlen játék!", "Játék vége!", "OK");
        _gameModel.NewGame(); // új játékot kezdünk
    }

    #endregion

    #region ViewModel event handlers

    /// <summary>
    /// Játék betöltésének eseménykezelője.
    /// </summary>
    private async void GameViewModel_LoadGame(object? sender, System.EventArgs e)
    {
        await _storeModel.UpdateAsync(); // frissítjük a tárolt játékok listáját
        await Navigation.PushAsync(_loadGamePage); // átnavigálunk a lapra
    }

    /// <summary>
    /// Játék mentésének eseménykezelője.
    /// </summary>
    private async void GameViewModel_SaveGame(object? sender, System.EventArgs e)
    {
        await _storeModel.UpdateAsync(); // frissítjük a tárolt játékok listáját
        await Navigation.PushAsync(_saveGamePage); // átnavigálunk a lapra
    }

    /// <summary>
    /// Betöltés végrehajtásának eseménykezelője.
    /// </summary>
    private async void StoreViewModel_GameLoading(object? sender, StoredGameEventArgs e)
    {
        await Navigation.PopAsync(); // visszanavigálunk

        try
        {
            await _gameModel.LoadGameAsync(Path.Combine(FileSystem.AppDataDirectory, e.Name)); // betöltjük a megadott játékot
        }
        catch
        {
            await DisplayAlert("Hiba!", "A betöltés sikertelen volt.", "OK");
        }
    }

    /// <summary>
    /// Mentés végrehajtásának eseménykezelője.
    /// </summary>
    private async void StoreViewModel_GameSaving(object? sender, StoredGameEventArgs e)
    {
        await Navigation.PopAsync(); // visszanavigálunk

        try
        {
            await _gameModel.SaveGameAsync(Path.Combine(FileSystem.AppDataDirectory, e.Name)); // lementjük a megadott játékot
            await _storeModel.UpdateAsync();  // frissítjük a tárolt játékok listáját
        }
        catch
        {
            await DisplayAlert("Hiba!", "A mentés sikertelen volt.", "OK");
        }
    }

    #endregion
}
